#[non_exhaustive]pub enum Error {
InvalidVectorLength,
InvalidVectorMagnitude,
InvalidQuaternionMagnitude,
}Expand description
Enumerate possible sources of error in fallible vector math operations.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
InvalidVectorLength
Attempted converting a value to a vector with a dimension not equal to the value’s length.
InvalidVectorMagnitude
Attempted to normalize a vector with an invalid magnitude.
InvalidQuaternionMagnitude
Attempted to normalize a quaternion with an invalid magnitude.