Angle

Struct Angle 

pub struct Angle {
    pub theta: f64,
}
Rotation in the plane.

The rotation is represented by an angle theta in radians. Positive values rotate counter-clockwise.

Constructing Angle

The default Angle rotates by 0 radians:

use hoomd_vector::Angle;

let a = Angle::default();
assert_eq!(a.theta, 0.0)

Create an Angle with a given value:

use hoomd_vector::Angle;
use std::f64::consts::PI;

let a = Angle::from(PI / 2.0);
assert_eq!(a.theta, PI / 2.0);

Create a random Angle from the uniform distribution over all rotations:

use hoomd_vector::Angle;
use rand::{RngExt, SeedableRng, rngs::StdRng};

let mut rng = StdRng::seed_from_u64(1);
let a: Angle = rng.random();

Operations using Angle

Rotate a Cartesian<2> vector by an Angle:

use approxim::assert_relative_eq;
use hoomd_vector::{Angle, Cartesian, Rotate, Rotation};
use std::f64::consts::PI;

let v = Cartesian::from([-1.0, 0.0]);
let a = Angle::from(PI / 2.0);
let rotated = a.rotate(&v);
assert_relative_eq!(rotated, [0.0, -1.0].into())

Combine two rotations together:

use hoomd_vector::{Angle, Rotation};
use std::f64::consts::PI;

let a = Angle::from(PI / 2.0);
let b = Angle::from(-PI / 4.0);
let c = a.combine(&b);
assert_eq!(c.theta, PI / 4.0);

Fields

theta: f64

Rotation angle (radians).

Implementations

Source

impl Angle

pub fn to_reduced(self) -> Self

Reduce the rotation.

Angle rotations are well-defined for any value of theta. However, combining small rotations with large ones will introduce floating point round-off error. Reducing an Angle creates an equivalent rotation with theta in the range from 0 to 2 pi.

Example
use hoomd_vector::Angle;
use std::f64::consts::PI;

let a = Angle::from(20.0 * PI);
let b = a.to_reduced();
assert_eq!(b.theta, 0.0)