Versor

Struct Versor 

pub struct Versor(/* private fields */);
A unit Quaternion that represents a 3D rotation.

Versor represents a 3D rotation with a unit quaternion. Rotation follows the Hamilton convention.

Constructing a Versor:

The default Versor is the identity:

use hoomd_vector::Versor;

let v = Versor::default();
assert_eq!(*v.get(), [1.0, 0.0, 0.0, 0.0].into());

Create a Versor that rotates by an angle about an axis:

use hoomd_vector::Versor;
use std::f64::consts::PI;

let v = Versor::from_axis_angle([0.0, 1.0, 0.0].try_into()?, PI / 2.0);
assert_eq!(
    *v.get(),
    [(PI / 4.0).cos(), 0.0, (PI / 4.0).sin(), 0.0].into()
);

Create a Versor by normalizing a Quaternion:

use hoomd_vector::{Quaternion, Versor};

let q = Quaternion::from([3.0, 0.0, 0.0, 4.0]);
let v = q.to_versor()?;
assert_eq!(*v.get(), [3.0 / 5.0, 0.0, 0.0, 4.0 / 5.0].into());

Create a random Versor:

use hoomd_vector::Versor;
use rand::{RngExt, SeedableRng, rngs::StdRng};

let mut rng = StdRng::seed_from_u64(1);
let v: Versor = rng.random();

Operations using Versor

Rotate a Cartesian<3> by a Versor:

use approxim::assert_relative_eq;
use hoomd_vector::{Cartesian, Rotate, Rotation, Versor};
use std::f64::consts::PI;

let a = Cartesian::from([-1.0, 0.0, 0.0]);
let v = Versor::from_axis_angle([0.0, 0.0, 1.0].try_into()?, PI / 2.0);
let b = v.rotate(&a);
assert_relative_eq!(b, [0.0, -1.0, 0.0].into());

Combine two rotations together:

use hoomd_vector::{Rotation, Versor};
use std::f64::consts::PI;

let a = Versor::from_axis_angle([1.0, 0.0, 1.0].try_into()?, PI / 2.0);
let b = Versor::from_axis_angle([0.0, 0.0, 1.0].try_into()?, PI / 4.0);
let c = a.combine(&b);

Implementations

Source

impl Versor

pub fn from_axis_angle(axis: Unit<Cartesian<3>>, angle: f64) -> Self

Create a Versor that rotates by an angle (in radians) counterclockwise about an axis.

Example
use hoomd_vector::Versor;
use std::f64::consts::PI;

let v = Versor::from_axis_angle([0.0, 1.0, 0.0].try_into()?, PI / 2.0);
assert_eq!(
    *v.get(),
    [(PI / 4.0).cos(), 0.0, (PI / 4.0).sin(), 0.0].into()
);
pub fn normalized(self) -> Self

Normalize the versor.

Nominally, all Versor instances retain a unit norm. Due to limited floating point precision, this assumption may not hold after repeated operations. Normalize versors when needed to correct this issue.

Example
use hoomd_vector::Versor;
use std::f64::consts::PI;

let a = Versor::from_axis_angle([0.0, 1.0, 0.0].try_into()?, PI / 2.0);
let b = a.normalized();
pub fn get(&self) -> &Quaternion

Get the unit quaternion.

pub fn arc_distance(&self, other: &Self) -> f64

A metric quantifying the angle (in radians) of the spherical arc separating two Versors.

$d : \mathbb{H} \times \mathbb{H} \to \mathbb{R}^+, \quad d(q_0, q_1) = \arccos(|q_0 \cdot q_1|)$

This value always lies in the range $[0, \pi]$, and is symmetric: while there are multiple arcs separating a pair of quaternions, this metric always chooses the shortest.

pub fn half_euclidean_norm_squared(&self, other: &Self) -> f64

A fast metric on Versors representing elements of SO(3).

$d : \mathbb{H} \times \mathbb{H} \to \mathbb{R}^+, \quad d(q_0, q_1) = 1 - |q_0 \cdot q_1 |$

This has less geometric meaning than the arc_distance metric. However, it is much faster while still obeying the triangle inequality and the axiom $d(q_0, q_1) = d(q_1, q_0)$. This metric always lies in the range $[0, 1]$, and is symmetric such that $d(q, q)$ = $d(q, -q)$.