Cartesian

Struct Cartesian 

pub struct Cartesian<const N: usize> {
    pub coordinates: [f64; N],
}
A Vector represented by N f64 coordinates.

Cartesian is the canonical implementation of InnerProduct.

Constructing vectors

The default is the 0 vector:

use hoomd_vector::Cartesian;

let v = Cartesian::<3>::default();
assert_eq!(v, [0.0; 3].into())

Create a vector with an array of coordinates:

use hoomd_vector::Cartesian;

let v = Cartesian::from([1.0, 2.0, 3.0, 4.0, 5.0]);

2D and 3D vectors can also be initialized from tuples:

use hoomd_vector::Cartesian;

let a = Cartesian::from((1.0, 2.0, 3.0));
let b = Cartesian::from((4.0, 5.0));

Construct a random vector in the [-1, 1] hypercube:

use hoomd_vector::Cartesian;
use rand::{RngExt, SeedableRng, rngs::StdRng};

let mut rng = StdRng::seed_from_u64(1);
let v: Cartesian<3> = rng.random();

Operating on vectors

Use vector math operations when you can:

use hoomd_vector::{Cartesian, InnerProduct};

let a = Cartesian::from([1.0, 2.0]);
let b = Cartesian::from([4.0, 8.0]);
let c = (a + b).dot(&a);

Access the coordinates directly when needed:

use hoomd_vector::Cartesian;

let a = Cartesian::from((1.0, 2.0));
let b = Cartesian::from((a[1], 0.0));

Compute the sum of an iterator over vectors:

use hoomd_vector::Cartesian;

let total: Cartesian<2> =
    [Cartesian::from((1.0, 2.0)), Cartesian::from((3.0, 4.0))]
        .into_iter()
        .sum();

Fields

coordinates: [f64; N]

The vector’s coordinates.

Implementations

Source

impl Cartesian<4>

pub fn counary_cross(vectors: &[Self; 3]) -> Self

Compute the N-1-ary (co-unary) cross product of a four-dimensional vector.

This function is a generalization of the cross product to general dimension, identifying a unique vector perpendicular to N-1 other vectors. In two dimensions, this is Cartesian::<2>::perpendicular, and in three dimensions this is simply Cross.

In geometric algebra terms, this is the Hodge dual of the exterior (Wedge) product. Geometrically, the dot product of the resulting vector with all inputs is zero and the magnitude of the vector is the hypervolume of the parallelepiped spanned by the inputs.

Source

impl Cartesian<2>

pub fn perpendicular(self) -> Self

Construct a 2-vector perpendicular to self.

Given a vector $(v_x, v_y)$ perpendicular returns the vector rotated by $\pi/2$:

(-v_y, v_x)
Example
use hoomd_vector::Cartesian;

let v = Cartesian::from([1.0, -4.5]);
assert_eq!(v.perpendicular(), [4.5, 1.0].into());
Source

impl<const N: usize> Cartesian<N>

pub fn to_row_matrix(self) -> Matrix<1, N>

Convert a Cartesian<N> into a row matrix Matrix<1, N>.

Example
use hoomd_linear_algebra::matrix::Matrix;
use hoomd_vector::Cartesian;

let a = Cartesian::from([1.0, -2.0, 3.0]);

let b = a.to_row_matrix();
assert_eq!(b.rows, [[1.0, -2.0, 3.0]]);
pub fn to_column_matrix(self) -> Matrix<N, 1>

Convert a Cartesian<N> into a column matrix Matrix<N, 1>.

Example
use hoomd_linear_algebra::matrix::Matrix;
use hoomd_vector::Cartesian;

let a = Cartesian::from([1.0, -2.0, 3.0]);

let b = a.to_column_matrix();
assert_eq!(b.rows, [[1.0], [-2.0], [3.0]]);